From e72f15d9e16437fdd26c95b6130b28219d9f279e Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?=E4=BD=99=E6=98=8C=E5=B9=B3?= <dead.ash@hotmail.com>
Date: Thu, 18 Jun 2020 17:24:48 +0800
Subject: [PATCH V2 1/2] kernel32/tests: Add tests for getting main thread id 
by toolhelp.
MIME-Version: 1.0
Content-Type: text/plain; charset=UTF-8
Content-Transfer-Encoding: 8bit

Some software incorrectly uses the way to get the main thread id. 
Coincidentally, the threads of wine and windows return differently.
The list of threads returned by flipping can be consistent with windows.

Signed-off-by: Changping Yu <dead.ash@hotmail.com>
---
 dlls/kernel32/tests/toolhelp.c | 67 ++++++++++++++++++++++++++++++++++
 1 file changed, 67 insertions(+)

diff --git a/dlls/kernel32/tests/toolhelp.c b/dlls/kernel32/tests/toolhelp.c
index 9250f8dc54..257df34ce4 100644
--- a/dlls/kernel32/tests/toolhelp.c
+++ b/dlls/kernel32/tests/toolhelp.c
@@ -150,6 +150,72 @@ static void test_process(DWORD curr_pid, DWORD sub_pcs_pid)
     ok(!pProcess32First( hSnapshot, &pe ), "shouldn't return a process\n");
 }
 
+static DWORD WINAPI get_thread(void* curr_pid)
+{
+    HANDLE              hSnapshot;
+    THREADENTRY32       te;
+    HANDLE              ev, hThread[10];
+    DWORD               tid, first_tid = 0;
+    BOOL                found = FALSE;
+
+    ev = CreateEventW(NULL, FALSE, FALSE, NULL);
+    ok(ev != NULL, "Cannot create event\n");
+
+    for(int i = 0; i < 10; i++)
+        hThread[i] = CreateThread(NULL, 0, sub_thread, ev, 0, &tid);
+
+    ok(hThread != NULL, "Cannot create thread\n");
+
+    hSnapshot = pCreateToolhelp32Snapshot( TH32CS_SNAPTHREAD, 0 );
+    ok(hSnapshot != NULL, "Cannot create snapshot\n");
+
+    /* check that this current process is enumerated */
+    te.dwSize = sizeof(te);
+    if (pThread32First( hSnapshot, &te ))
+    {
+        do
+        {
+            /* first matched thread id */
+            if (te.th32OwnerProcessID == (DWORD)curr_pid){
+                found = TRUE;
+                first_tid = te.th32ThreadID;
+                break;
+            }
+        } while (pThread32Next( hSnapshot, &te ));
+    }
+
+    ok(found, "couldn't find self and/or sub-process in process list");
+
+    CreateThread(NULL, 0, sub_thread, ev, 0, &tid);
+
+    SetEvent(ev);
+    for(int i = 0; i < 10; i++)
+        CloseHandle(hThread[i]);
+    CloseHandle(ev);
+    CloseHandle(hSnapshot);
+
+    return first_tid;
+}
+
+static void test_main_thread(DWORD curr_pid, DWORD main_tid)
+{
+    HANDLE              hThread;
+    DWORD               tid, first_tid = 0;
+    int                 error;
+
+    first_tid = get_thread((void *)curr_pid);
+    todo_wine
+    ok(first_tid == main_tid, "check main thread is error, main is %d, %d\n", main_tid, first_tid);
+
+    hThread = CreateThread(NULL, 0, get_thread, (void *)curr_pid, 0, &tid);
+    error = WaitForSingleObject(hThread, 5000);
+    ok(error==WAIT_OBJECT_0, "Thread did not complete within timelimit\n");
+
+    ok(GetExitCodeThread(hThread, &first_tid), "Could not retrieve ext code\n");;
+    todo_wine
+    ok(first_tid == main_tid, "check main thread is error, main is %d, %d\n", main_tid, first_tid);
+}
+
 static void test_thread(DWORD curr_pid, DWORD sub_pcs_pid)
 {
     HANDLE              hSnapshot;
@@ -339,6 +405,7 @@ START_TEST(toolhelp)
 
     test_process(pid, info.dwProcessId);
     test_thread(pid, info.dwProcessId);
+    test_main_thread(pid, GetCurrentThreadId());
     test_module(pid, curr_expected_modules, ARRAY_SIZE(curr_expected_modules));
     test_module(info.dwProcessId, sub_expected_modules, ARRAY_SIZE(sub_expected_modules));
 
-- 
2.26.1.windows.1

